<?php

namespace Modules\Manager\Http\Controllers;

use Illuminate\Http\Request;
use App\Models\Role;
use Illuminate\Support\Facades\Log;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Validator;

class RoleController extends Controller
{
    //
/* 获取角色信息*/
    public function getRolesList(){
        $list=Role::orderBy('id', 'desc')->get();
        if($list){
            return $this->success($list);
        }
        return $this->error();
    }
/*添加 角色*/
    public function uploadRole(Request $request){
        $rules=[
            'name' => 'required|max:32|unique:roles',
            'permissions' => 'required',
        ];
        $messages=[
            'name.required'=>'角色名称不能为空.',
            'name.max'=>'角色名称不能超过32个字符.',
            'name.unique'=>'角色名称已存在.',
            'permissions.required'=>'请选择角色权限.',
        ];
        $validator = Validator::make($request->all(),$rules,$messages);
        if ($validator->fails()) {
            return $this->error('400013',$validator->errors()->first());
        }
        $result=Role::create([
            'name'=>$request->post('name'),
            'permissions'=>$request->post('permissions'),
            'desc'=>$request->post('desc'),
            'disable'=>0,
        ]);
        if($result->id){
            Log::info('管理员:'.Auth::user()->name.'(id='.Auth::user()->id.')添加（'.$request->post('name').'）角色成功');
            return $this->success([]);
        }
        return $this->error();
    }
/* 角色禁用  */
    public function disableRole(Request $request){
        $role=Role::find($request->post('id'));
        $role->disable=($role->disable==0)?'1':'0';
        $row=$role->save();
        if($row){
            Log::info('管理员:'.Auth::user()->name.'(id='.Auth::user()->id.')禁用了（'.$role->name.'）角色');
            return $this->success([]);
        }
        return $this->error();
    }
    /*修改角色*/
    public function updateRole(Request $request){
        $input=$request->all();
        $rules=[
            'name' => 'required|max:32',
            'permissions' => 'required',
        ];
        $messages=[
            'name.required'=>'角色名称不能为空.',
            'name.max'=>'角色名称不能超过32个字符.',
            'permissions.required'=>'请选择角色权限.',
        ];
        $validator = Validator::make($input,$rules,$messages);
        if ($validator->fails()) {
            return $this->error('400013',$validator->errors()->first());
        }
        $role=Role::where('name',$input['name'])->first();
        if($role){
            if($role->id!=$input['id']){
                return $this->error('400012','角色名称已存在.');
            }
        }
        $roles=Role::find($request->id);
        $roles->name=$request->name;
        $roles->permissions=$request->permissions;
        $roles->desc=$request->desc;
        $result=$roles->save();
        if($result){
            Log::info('管理员:'.Auth::user()->name.'(id='.Auth::user()->id.')修改了（'.$role->name.'）角色');
            return $this->success([]);
        }
        return $this->error();
    }
}
