<?php

namespace Modules\Manager\Jobs\Trace;

use App\Models\Serial;
use App\Models\TraceTest;
use Illuminate\Bus\Queueable;
use Illuminate\Queue\SerializesModels;
use Illuminate\Queue\InteractsWithQueue;
use Illuminate\Contracts\Queue\ShouldQueue;
use Illuminate\Foundation\Bus\Dispatchable;
use Rap2hpoutre\FastExcel\FastExcel;
use Illuminate\Support\Facades\Log;

class DownloadFileJob implements ShouldQueue
{
    use Dispatchable, InteractsWithQueue, Queueable, SerializesModels;

    public $tries = 5;
//    public $maxExceptions = 3;
    public $timeout = 60;
    /**
     * Create a new job instance.
     *
     * @return void
     */
    protected $data;
    public function __construct($data)
    {
        $this->data=$data;
    }

    /**
     * Execute the job.
     *
     * @return void
     */
    public function handle()
    {
        $serial=$this->data;
        $model=$serial->model;
        $new_model=str_replace(":","_",$model);
        $sort=$serial->sort;
        $path=storage_path().'/app/public/trace/model'.$new_model.'/';
        $filename = '第'.$sort.'批-类型'.$new_model.'-数量'.$serial->num.'组.csv';
        $file =$path.$filename;
        (new FastExcel($this->usersGeneratorFastExcel($model,$sort)))
            ->export($file, function ($order) {
                return [
                    '大码' => trim($order['max_code']),
                    '中码' => trim($order['sec_code']),
                    '小码' => trim($order['code']),
                    '防伪码' => $order['security_code'],
                    '类型' => $order['model'],
                ];
            });
        Serial::where('id',$serial->id)->update([
            'down_status'=>2,
            'down_url'=>env('APP_URL').'/storage/trace/model'.$new_model.'/'.$filename
        ]);
    }

    function usersGeneratorFastExcel($model,$sort) {
        $data=TraceTest::where('model',$model)->where('serial',$sort)->where('is_lowest',1)
            ->select('model','security_code','code')->cursor();
        foreach ($data as $value) {
            yield $value;
        }
    }
}
