<?php

namespace Modules\Service\Listeners;

use Modules\Service\Events\SendApplyResultSmsEvent;
use App\Handlers\JudgeLevelNameHandler;
use App\Models\ApplyUser;
use Carbon\Carbon;
use Illuminate\Queue\InteractsWithQueue;
use Illuminate\Contracts\Queue\ShouldQueue;
use Illuminate\Support\Facades\Log;
use Overtrue\EasySms\EasySms;

class SendApplyResultSmsListener
{
    /**
     * Create the event listener.
     *
     * @return void
     */
    public function __construct()
    {
        //
    }

    /**
     * Handle the event.
     *
     * @param  object  $event
     * @return void
     */
    public function handle(SendApplyResultSmsEvent $event)
    {
        $data=$event->data;
        $apply=ApplyUser::where('id',$data['id'])->with(['GetApplyUser'])->with(['GetUser'])->first()->toArray();
        if (isset($apply['get_apply_user'])){
            if (isset($apply['get_apply_user']['nickname'])){
                $nickname=$apply['get_apply_user']['nickname'];
            }elseif ($apply['get_apply_user']['realname']){
                $nickname=$apply['get_apply_user']['realname'];
            }else{
                Log::error('发送审核短信没找到对应的信息1，审核ID为：'.$data['id']);
                return;
            }
            $level=$apply['get_apply_user']['level'];
            $mobile=$apply['get_user']['mobile'];
        }else{
            Log::error('发送审核短信没找到对应的信息，审核ID为：'.$data['id']);
            return;
        }
        try{
            $easySms=new EasySms(config('easysms'));
            $content="【大卫博士】".$this->SearchSmsContentStatus($apply['status'],$apply['type'],$nickname,$level);
            $easySms->send($mobile, [
                'content' =>$content
            ]);
        }catch (\Exception $exception){
            \Log::error($exception->getMessage());
        }
    }

    public function SearchSmsContentType($type){
        switch ($type){
            case 0:
                $text='降级';
                break;
            case 1:
                $text='升级';
                break;
            case 2:
                $text='删除';
                break;
            default:
                $text='';
                break;
        }
        return $text;
    }

    public function SearchSmsContentStatus($status,$type,$nickname,$level){
        $handler=new JudgeLevelNameHandler();
        $date=Carbon::now()->format('Ymd');
        $year=mb_substr($date,0,4);
        $month=mb_substr($date,4,2);
        $day=mb_substr($date,6,2);
        switch ($status){
            case 1;
                $text=$this->SearchSmsContentType($type);
                if ($type==0){
                    $level=2;
                }elseif ($type==1){
                    $level=1;
                }
                $levelName=$handler->JudgeLevel($level);
                $content="您于{$year}年{$month}月{$day}日提交的关于{$nickname}的申请，已通过审核。该{$levelName} 已经{$text}。";
                break;
            case 2;
                $content="您于{$year}年{$month}月{$day}日提交的关于{$nickname}的申请，已被驳回。如有异议，请联系政委。";
                break;
        }
        return $content;
    }
}
